{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {
    "tags": [
     "remove_cell"
    ]
   },
   "outputs": [],
   "source": [
    "from IPython.display import display, IFrame\n",
    "def example(url): display(IFrame(url, width=\"100%\", height=600))"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Теоретичні відомості\n",
    "*Список* — це послідовність структур, кожна з яких містить посилання, яке зв’язує її з іншою структурою. Для організації списків використовують структури, які складаються з двох частин — інформаційної та допоміжної:\n",
    "- інформаційна частина містить інформацію, яка підлягає опрацюванню;\n",
    "- додаткова частина містить указівники на наступну чи попередню структури списку.\n",
    "\n",
    "Елемент списку називають *вузлом* (node). Таким чином, список — це ланцюжок зв’язаних вузлів, від першого до останнього. Останній вузол не посилається на наступний елемент, тому його поле посилання має значення `NULL`. За однозв’язним списком можна рухатися тільки в одному напрямку — від заголовкового (першого, root) вузла до останнього.\n",
    "\n",
    "У даній лабораторній роботі для реалізації списку використовуватимемо структуру, яку називають *зв’язним списком* (linked list). Кожний його вузол містить поле даних `data` і вказівник на наступний вузол `next` (рисунок 7.1).\n",
    "\n",
    "![Структура зв’язного списку](../images/lab71.png)\n",
    " \n",
    "Рисунок 7.1 – Структура зв’язного списку\n",
    "\n",
    "Перший елемент списку називаються *головою* (head), у роботі адресуватимемо його вказівником `head`. Останній елемент називаються *хвостом* (tail). Поле `next` хвоста списку має значення `NULL`. Програми, які працюють із таким списком, послідовно проходять вузлами, використовуючи вказівники `next`, починаючи з голови до хвоста, ознакою якого є `next == NULL`.\n",
    "\n",
    "На відміну від масиву, зв’язний список не вимагає безперервної області пам’яті: вузли можуть розташовуватися в ній довільним чином. Найчастіше вузли зв’язного списку є динамічними об’єктами, які створюють за потребою за допомогою функцій `malloc()` чи `calloc()` і знищують за допомогою функції `free()`.\n",
    "\n",
    "Процедури додавання й видалення вузлів зв’язного списку не вимагають копіювання даних, а обмежуються тільки зміною вказівників. Вставка вузла у зв’язний список зводиться до «налаштування» вказівників next двох вузлів: вставленого й попереднього. Видалення вузла полягає лише в зміні поля next вузла, попереднього тому вузлу, що видаляється.\n",
    "\n",
    "Розгляньмо спосіб опису структури однозв’язного списку:\n",
    "```c\n",
    "struct list{\n",
    "    char data[20];\n",
    "    struct list* next; /*указівник на структуру*/\n",
    "};\n",
    "```\n",
    "\n",
    "Для опису вказівника використовується ще не описаний об’єкт `struct list* next`, який слугуватиме посиланням на наступний елемент списку. Найостанніший вузол у списку нікуди не вказує, тобто поле next повинно мати значення `NULL`.\n",
    "\n",
    "Виділяють типові операції над списками:\n",
    "- додавання вузла в початок списку;\n",
    "- видалення вузла з початку списку;\n",
    "- додавання вузла в довільне місце списку, відмінне від початку (наприклад, після вузла, указівник на який задано);\n",
    "- видалення вузла з довільного місця списку, відмінного від початку (наприклад, після вузла, указівник на який задано);\n",
    "- перевірка, чи порожній список;\n",
    "- очищення списку;\n",
    "- друк списку.\n",
    "\n",
    "Цей набір операцій рекомендується реалізувати у вигляді окремого модуля. Підключивши цей модуль, можна вирішити більшість типових завдань."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "```{margin}\n",
    "Запустити цей приклад можна за [посиланням](https://repl.it/@GasperPaul/Lab7-Example-71).\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Приклад 7.1\n",
    "У нижченаведеній програмі продемонстровано створення й перегляд однозв’язного списку.\n",
    "\n",
    "```c\n",
    "#include <stdio.h> \n",
    "#include <stdlib.h>\n",
    "\n",
    "typedef struct list {\n",
    "    char data[20];\n",
    "    struct list* next;\n",
    "} list;\n",
    "\n",
    "/* функція створення списку */\n",
    "struct list* create(void);\n",
    "/* функція перегляду списку */\n",
    "void show(list* head);\n",
    "/* функція звільнення пам'яті */\n",
    "void free_list(list* head);\n",
    "\n",
    "int main(void) \n",
    "{\n",
    "    list* head = create();\n",
    "    show(head);\n",
    "    free_list(head);\n",
    "}\n",
    "\n",
    "struct list* create(void)\n",
    "{\n",
    "    /* змінна адреси голови списку */\n",
    "    list* head;\n",
    "    /* р    - указівник на поточну структуру,\n",
    "       prev - указівник на попередню структуру */\n",
    "    list *p, *prev; \n",
    "\n",
    "    char ch;\n",
    "    do {\n",
    "        p = (list*) malloc(sizeof(list));\n",
    "        if (!p) exit(1);\n",
    "\n",
    "        // якщо це перший запис - це голова\n",
    "        if (!head)\n",
    "            head = prev = p;\n",
    "\n",
    "        printf(\"\\nУведіть ім'я: \");\n",
    "        scanf(\"%s\", p->data);\n",
    "\n",
    "        if (prev) {\n",
    "            prev->next = p;\n",
    "        }\n",
    "        prev = p;\n",
    "\n",
    "        printf(\"Закінчити? y/n\");\n",
    "        scanf(\" %c\", &ch);\n",
    "    } while (ch != 'y');\n",
    "\n",
    "    p->next = NULL;\n",
    "\n",
    "    return head;\n",
    "}\n",
    "\n",
    "void show(list* head)\n",
    "{\n",
    "    // указівник, яким проходитимемо список \n",
    "    list* p = head;\n",
    "\n",
    "    while (p) {\n",
    "        /* поки не кінець списку */\n",
    "        printf(\"\\nІм'я: %s\", p->data);\n",
    "        /* переходимо на наступний елемент списку */\n",
    "        p = p->next;\n",
    "    }\n",
    "}\n",
    "\n",
    "void free_list(list* head)\n",
    "{\n",
    "    list *p;\n",
    "    \n",
    "    while (head) {\n",
    "        p = head;\n",
    "        head = head->next;\n",
    "        free(p);\n",
    "    } \n",
    "}\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Добрий приклад роботи зі структурою списку представлено в {cite}`deitel`."
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.6.3"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
